//----------------------------------------------------------------------------
//
//       FILENAME: NonVolatile.c
//
//    DESCRIPTION: Non Volatile Flash memory functions
//
//  LAST MODIFIED: Oct 23, 2006
//
//-----------------------------------------------------------------------------
//  Copyright (c) Toradex AG 2006. All Rights Reserved.
//----------------------------------------------------------------------------
#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "flashtemp.h"	// Measuring the Flash temperature
#include "flashblock.h"	// Cypress API for reading and writing flash blocks

BYTE				RWBuf[66];	// holds the Flash Block for modification
FLASH_WRITE_STRUCT	fwStruct;
FLASH_READ_STRUCT	frStruct;
//-----------------------------------------------------------------------------
// Modify Flash contents within a single flash block
//	- Temperature (in C) is necessary for Flash programming. 
//	  Accuracy of +-25C is sufficient
//  - return value is 0 if operation there was an error
//
// NOTE: edit "flashsecurity.txt" to ensure the modified flash block is
//       not write-protected!
//-----------------------------------------------------------------------------
BYTE Flash_ModifyBlk (BYTE FlashBlk, BYTE BlkOffset, BYTE *src, BYTE Length, BYTE Temperature)
{
	BYTE Result;
#ifdef __DEBUG
	// fill remaining stack with 0xAA (to test for stack overflow)
	asm (
		"PUSH	X		\n"
		"PUSH	A		\n"
		
		"MOV	X, SP	\n"
		"INC	X		\n"
		"MOV	A, 0xAA	\n"
	".LOOP:				\n"
		"MOV	[X], A	\n"
		"INC	X		\n"		
		"JNZ	.LOOP	\n"

		"POP	A		\n"
		"POP	X		\n"
		);
#endif		
	// Range check for "BlkOffset" and "Length" parameter
	if ( (BlkOffset + Length) >64)
		return 0;
	
	// Read complete Flash Block into RAM buffer
	frStruct.wARG_BlockId     = FlashBlk;
	frStruct.pARG_FlashBuffer = RWBuf;
	frStruct.wARG_ReadCount   = 64;
	FlashReadBlock( &frStruct );
	
	// ---- change Flash contents in RAM
	while (Length > 0)
	{
		RWBuf[BlkOffset] = *src;
		
		Length--;
		src++;
		BlkOffset++;
	}

	// ---- write Flash block back into FLASH
	fwStruct.wARG_BlockId     = FlashBlk;
	fwStruct.pARG_FlashBuffer = RWBuf;
	fwStruct.cARG_Temperature = Temperature;  
#ifdef __DEBUG__
M8C_DisableGInt;
#endif
	Result = bFlashWriteBlock( &fwStruct );
#ifdef __DEBUG__
M8C_EnableGInt;
#endif

	return Result;
}


//-----------------------------------------------------------------------------
// change Flash memory content
//  - data may be split across multiple flash blocks (a block is 64 Bytes)
//  - return value is 0 if there was an error
//-----------------------------------------------------------------------------
BYTE Flash_Modify (const BYTE *dest, BYTE *src, BYTE length)
{
	BYTE	FlashBlk, FlashBlk_End, BlkOffset;
	BYTE	WrLength;
	BYTE	Result;
	char	Temperature;
		
	// ----- update Flash temperature
	FlashTemp_Start();
	while (!FlashTemp_fIsData());
	Temperature = FlashTemp_cGetData();
	FlashTemp_Stop();
	
	// ----- calculate Block and offset
	FlashBlk       = ( (WORD)dest << 2 ) /256;
	FlashBlk_End   = ( (WORD)(dest+length-1) << 2 ) /256;
	BlkOffset      = (WORD)dest & 0x3F;
	
	// ----- loop once for every flash block to modify
	while (length > 0)  
	{
		// -- adapt length if this is not the last block
		if (FlashBlk < FlashBlk_End)
			WrLength = 64 - BlkOffset;
		else
			WrLength = length;
		
		// -- change flash
		Result = Flash_ModifyBlk (FlashBlk, BlkOffset, src, WrLength, Temperature);
		if (Result == 0)
			return 0;	// error occured
			
		// -- prepare for next block to modify
		src      += WrLength;  
		length   -= WrLength;
		FlashBlk++;
		BlkOffset = 0;
	}
	
	return 1;
}